"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const middleware_1 = require("./middleware");
describe("Middleware", () => {
    const authMiddleware = middleware_1.Middleware.of((req, original, handler) => {
        const userId = original.originalEvent.headers["x-user-id"];
        if (!userId) {
            return Promise.resolve({
                statusCode: 401,
                body: "Unauthorized",
            });
        }
        else {
            return handler(req, original, { userId });
        }
    });
    const routes = __1.LambdaRouter.build((r) => r.get("/route")(authMiddleware(async (req, orig, user) => ({
        statusCode: 200,
        body: user,
    }))));
    const act = (headers) => routes({
        path: "/route",
        httpMethod: "GET",
        headers,
    }, null, null);
    it("should inject variables into wrapped route", async () => {
        const response = await act({
            "x-user-id": "123",
        });
        expect(response.statusCode).toEqual(200);
        expect(JSON.parse(response.body)).toEqual({ userId: "123" });
    });
    it("should intercept route where required", async () => {
        const response = await act({});
        expect(response.statusCode).toEqual(401);
    });
});
