import { RouteHandlerDefinition } from "./router";
export declare type ApiInfo = {
    title: string;
    description?: string;
    version: string;
};
declare type ApiGatewayOpenIdAuthorizerConfiguration = {
    type: "jwt";
    jwtConfiguration: {
        audience: string[];
    };
    identitySource: string;
};
declare type ApiGatewayOAuthAuthorizerConfiguration = {
    type: "jwt";
    jwtConfiguration: {
        issuer: string;
        audience: string[];
    };
};
declare type CustomLambdaAuthorizerConfiguration = {
    type: "request" | "token";
    identitySource?: "$request.header.Authorization" | string;
    authorizerUri: string;
    authorizerPayloadFormatVersion: "1.0" | "2.0";
    authorizerResultTtlInSeconds: number;
    enableSimpleResponses?: boolean;
};
declare type OpenIdSecurityScheme = {
    type: "openIdConnect";
    openIdConnectUrl: string;
    "x-amazon-apigateway-authorizer": ApiGatewayOpenIdAuthorizerConfiguration;
};
declare type OAuthSecurityScheme = {
    type: "oauth2";
    "x-amazon-apigateway-authorizer": ApiGatewayOAuthAuthorizerConfiguration;
};
declare type CustomLambdaSecurityScheme = {
    type: "apiKey";
    in: "header" | "query";
    name: "Authorization";
    "x-amazon-apigateway-authorizer": CustomLambdaAuthorizerConfiguration;
};
export declare type SecurityScheme = OpenIdSecurityScheme | OAuthSecurityScheme | CustomLambdaSecurityScheme;
export declare type RouteDefinition = Omit<RouteHandlerDefinition<"V1">, "handler">;
export declare const toOpenApi: (routes: readonly RouteDefinition[], payloadFormatVersion: "1.0" | "2.0", apiInfo: ApiInfo, functionArn: string, apiRoleArn?: string | undefined, securitySchemes?: {
    [k: string]: SecurityScheme;
} | undefined) => object;
export {};
