"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toOpenApi = void 0;
const typebox_1 = require("@sinclair/typebox");
const string_prototype_matchall_1 = __importDefault(require("string.prototype.matchall"));
const lodash_1 = require("lodash");
const paramRegex = /{(?<name>[\w-_]+)(:(?<type>\w+))?(?<optional>\?)?}/g;
const mapToOpenApiType = (s) => {
    switch (s) {
        case "int":
            return "integer";
        case "bool":
            return "boolean";
        default:
            return "string";
    }
};
const stripTypes = (p) => p.replace(/:\w+}/g, "}");
const toOpenApiObject = (route, payloadFormatVersion, apiInfo, functionArn, apiRoleArn, securitySchemes) => {
    const model = route.body ? typebox_1.Type.Strict(route.body) : null;
    const responses = Object.keys(route.responses).reduce((p, k) => {
        var _a, _b;
        return ({
            [parseInt(k)]: {
                description: ((_b = (_a = route.responses[parseInt(k)]) === null || _a === void 0 ? void 0 : _a.$static) === null || _b === void 0 ? void 0 : _b.description) || "",
                content: {
                    "application/json": {
                        schema: typebox_1.Type.Strict(route.responses[parseInt(k)]),
                    },
                },
            },
            ...p,
        });
    }, {});
    const [path, ...querys] = route.url.split("?");
    const query = querys.join("");
    const params = [...string_prototype_matchall_1.default(path, paramRegex)].map((p) => ({
        in: "path",
        name: p.groups["name"],
        required: true,
        schema: {
            type: mapToOpenApiType(p.groups["type"]),
        },
    }));
    const queryParams = query
        ? [...string_prototype_matchall_1.default(query, paramRegex)].map((p) => ({
            in: "query",
            name: p.groups["name"],
            required: !!p.groups["optional"],
            schema: {
                type: mapToOpenApiType(p.groups["type"]),
            },
        }))
        : null;
    const allParams = params.concat(queryParams || []);
    const paths = {
        [stripTypes(path)]: {
            [route.method.toLowerCase()]: {
                parameters: allParams,
                responses: responses,
                ...(model
                    ? {
                        requestBody: {
                            required: true,
                            content: {
                                "application/json": {
                                    schema: model,
                                },
                            },
                        },
                    }
                    : {}),
                "x-amazon-apigateway-integration": {
                    type: "AWS_PROXY",
                    httpMethod: "POST",
                    uri: functionArn,
                    payloadFormatVersion,
                    ...(apiRoleArn ? { credentials: apiRoleArn } : {}),
                },
                ...(route.useIamAuth
                    ? { "x-amazon-apigateway-auth": { type: "AWS_IAM" } }
                    : {}),
                ...(route.security
                    ? { security: [{ [route.security.scheme]: route.security.scopes }] }
                    : {}),
            },
        },
    };
    return {
        openapi: "3.0.1",
        info: apiInfo,
        paths,
        ...(securitySchemes
            ? {
                components: { securitySchemes },
            }
            : {}),
    };
};
const toOpenApi = (routes, payloadFormatVersion, apiInfo, functionArn, apiRoleArn, securitySchemes) => routes.reduce((p, n) => lodash_1.merge(p, toOpenApiObject(n, payloadFormatVersion, apiInfo, functionArn, apiRoleArn, securitySchemes)), {});
exports.toOpenApi = toOpenApi;
