"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const open_api_1 = require("./open-api");
const typebox_1 = require("@sinclair/typebox");
describe("toOpenApiPart", () => {
    it("should return a valid open api", () => {
        const api = open_api_1.toOpenApi([
            {
                url: "/people/{name:string}",
                method: "delete",
                useIamAuth: true,
                responses: {
                    200: typebox_1.Type.String(),
                },
            },
            {
                url: "/people/{name:string}",
                method: "get",
                useIamAuth: true,
                responses: {
                    200: typebox_1.Type.String(),
                },
            },
            {
                url: "/people/{name:string}/aged/{age:int}?{menOnly:bool?}",
                method: "post",
                useIamAuth: false,
                security: {
                    scheme: "jwt-authorizer-autofind",
                    scopes: [],
                },
                responses: {
                    200: typebox_1.Type.String(),
                },
                body: typebox_1.Type.Object({
                    name: typebox_1.Type.String({ description: "The Name" }),
                }, { additionalProperties: false }),
            },
        ], "1.0", { title: "Test Api", version: "1.0.0" }, "<arn placeholder>", 'arn:aws:iam:ap-southeast-2:123456789:my-role', {
            "jwt-authorizer-autofind": {
                type: "openIdConnect",
                openIdConnectUrl: "https://cognito-idp.region.amazonaws.com/userPoolId/.well-known/openid-configuration",
                "x-amazon-apigateway-authorizer": {
                    type: "jwt",
                    jwtConfiguration: {
                        audience: ["audience1", "audience2"],
                    },
                    identitySource: "$request.header.Authorization",
                },
            },
        });
        expect(api).toBeTruthy();
        console.log(JSON.stringify(api));
    });
});
