"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseQueryParams = exports.parsePathParams = exports.PathParamRegex = exports.PathParamParsers = void 0;
const string_prototype_matchall_1 = __importDefault(require("string.prototype.matchall"));
const Either_1 = require("fp-ts/lib/Either");
const E = __importStar(require("fp-ts/Either"));
const L = __importStar(require("fp-ts/Array"));
const FP = __importStar(require("fp-ts"));
const PaserErrorSemigroup = {
    concat: (a, b) => ({
        errors: a.errors.concat(b.errors),
    }),
};
class PathParamParsers {
    constructor() {
        this.string = {
            parse: (s) => Either_1.right(decodeURIComponent(s[0]))
        };
        this.int = {
            parse: (s) => {
                const result = parseInt(s[0]);
                return isNaN(result)
                    ? Either_1.left({ errors: [`Invalid int in path param: ${s}`] })
                    : Either_1.right(result);
            },
        };
        this.float = {
            parse: (s) => {
                const result = parseFloat(s[0]);
                return isNaN(result)
                    ? Either_1.left({ errors: [`Invalid float in path param: ${s}`] })
                    : Either_1.right(result);
            },
        };
        this["string[]"] = {
            parse: (s) => {
                return Either_1.right(s.map(decodeURIComponent));
            },
        };
        this["int[]"] = {
            parse: (s) => {
                const A = Either_1.getApplicativeValidation(PaserErrorSemigroup);
                return L.traverse(A)(this.float.parse)(s.map((i) => [i]));
            },
        };
        this["float[]"] = {
            parse: (s) => {
                const A = Either_1.getApplicativeValidation(PaserErrorSemigroup);
                return L.traverse(A)(this.float.parse)(s.map((i) => [i]));
            },
        };
    }
}
exports.PathParamParsers = PathParamParsers;
exports.PathParamRegex = /\{([^\}/]+)\}/;
const parsePathParams = (incomingUrl, pathSpec) => {
    const parsers = new PathParamParsers();
    const incomingPathSegments = incomingUrl.split("/");
    const pathSpecSegments = pathSpec.split("/");
    if (incomingPathSegments.length != pathSpecSegments.length) {
        return Either_1.left(["Invalid incoming url"]);
    }
    return pathSpecSegments.reduce((prev, n, ix) => {
        return FP.function.pipe(prev, E.chain((p) => {
            const params = exports.PathParamRegex.exec(n);
            if (params) {
                const param = params[1];
                const key = param.split(":")[0];
                const parserName = param.split(":")[1];
                const parser = parsers[parserName];
                const originalParam = incomingPathSegments[ix];
                const value = parser
                    ? parser.parse([originalParam])
                    : Either_1.right(originalParam);
                return FP.function.pipe(value, E.mapLeft((e) => e.errors), E.map((val) => ({
                    ...p,
                    [key]: val,
                })));
            }
            else {
                return Either_1.right(p);
            }
        }));
    }, Either_1.right({}));
};
exports.parsePathParams = parsePathParams;
const QueryParamRegex = /\{([^\}/]+)/g;
const parseQueryParams = (pathParams, multiPathParams, querySpec) => {
    const parsers = new PathParamParsers();
    return [...string_prototype_matchall_1.default(querySpec, QueryParamRegex)].reduce((p, n) => {
        return FP.function.pipe(p, FP.either.chain((p) => {
            const param = n[1];
            if (param) {
                const keySpec = param.split(":")[0];
                const key = keySpec.split("?")[0];
                const isNullable = keySpec.endsWith("?");
                const originalParam = pathParams[key] || multiPathParams[key];
                if (!isNullable && !originalParam) {
                    return FP.either.left([`Query param not found: ${key}`]);
                }
                else {
                    const parserName = param.split(":")[1];
                    const parser = parsers[parserName];
                    const value = parser && originalParam
                        ? parser.parse(Array.isArray(originalParam)
                            ? originalParam
                            : [originalParam])
                        : Either_1.right(originalParam);
                    return FP.function.pipe(value, E.mapLeft((e) => e.errors), E.map((val) => ({
                        ...p,
                        [key]: val,
                    })));
                }
            }
            else {
                return FP.either.right(p);
            }
        }));
    }, Either_1.right({}));
};
exports.parseQueryParams = parseQueryParams;
